@@ -23,7 +23,7 @@
 #include <resources/AbstractResourcesBackend.h>
 #include <resources/AbstractBackendUpdater.h>
 
-#include <QtCore/QStringBuilder>
+#include <QStringBuilder>
 #include <KLocalizedString>
 #include "libdiscover_debug.h"
 
@@ -47,11 +47,22 @@ QString UpdateItem::name() const
     return m_app->name();
 }
 
+// Deprecated; use availableVersion() instead
 QString UpdateItem::version() const
 {
     return m_app->availableVersion();
 }
 
+QString UpdateItem::availableVersion() const
+{
+    return m_app->availableVersion();
+}
+
+QString UpdateItem::installedVersion() const
+{
+    return m_app->installedVersion();
+}
+
 QVariant UpdateItem::icon() const
 {
     return m_app->icon();
