@@ -58,7 +58,6 @@
 
 #include <glib.h>
 #include <QRegularExpression>
-#include "FlatpakSourcesBackend.h"
 
 DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
@@ -153,21 +152,34 @@ public:
             const QUrl fileUrl = QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1Char('/') + originalUrl.fileName());
             auto replyPut = put(QNetworkRequest(fileUrl), replyGet->readAll());
             connect(replyPut, &QNetworkReply::finished, this, [this, originalUrl, fileUrl, replyPut]() {
-                if (replyPut->error() == QNetworkReply::NoError) {
-                    auto res = m_backend->resourceForFile(fileUrl);
-                    if (res) {
-                        FlatpakResource *resource = qobject_cast<FlatpakResource*>(res);
-                        resource->setResourceFile(originalUrl);
-                        Q_EMIT jobFinished(true, resource);
-                    } else {
-                        qWarning() << "couldn't create resource from" << fileUrl.toLocalFile();
-                        Q_EMIT jobFinished(false, nullptr);
-                    }
-                } else {
+                if (replyPut->error() != QNetworkReply::NoError) {
                     qWarning() << "couldn't save" << originalUrl << replyPut->errorString();
                     Q_EMIT jobFinished(false, nullptr);
+                    return;
                 }
-            });
+                if (!fileUrl.isLocalFile()) {
+                    Q_EMIT jobFinished(false, nullptr);
+                    return;
+                }
+
+                FlatpakResource *resource = nullptr;
+                if (fileUrl.path().endsWith(QLatin1String(".flatpak"))) {
+                    resource = m_backend->addAppFromFlatpakBundle(fileUrl);
+                } else if (fileUrl.path().endsWith(QLatin1String(".flatpakref"))) {
+                    resource = m_backend->addAppFromFlatpakRef(fileUrl);
+                } else if (fileUrl.path().endsWith(QLatin1String(".flatpakrepo"))) {
+                    resource = m_backend->addSourceFromFlatpakRepo(fileUrl);
+                }
+
+                if (resource) {
+                    resource->setResourceFile(originalUrl);
+                    Q_EMIT jobFinished(true, resource);
+                } else {
+                    qWarning() << "couldn't create resource from" << fileUrl.toLocalFile();
+                    Q_EMIT jobFinished(false, nullptr);
+                }
+            }
+            );
         });
     }
 
@@ -206,7 +218,7 @@ FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation
         return ref;
     }
 
-    const auto type = resource->type() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME;
+    const auto type = resource->resourceType() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME;
 
     return flatpak_installation_get_installed_ref(flatpakInstallation,
                                                  type,
@@ -371,6 +383,17 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
 {
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
     const QString refurl = settings.value(QStringLiteral("Flatpak Ref/Url")).toString();
+    const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
+
+    auto item = m_sources->sourceByUrl(refurl);
+    if (item) {
+        const auto resources = resourcesByAppstreamName(name);
+        for (auto resource : resources) {
+            if (resource->origin() == item->data(AbstractSourcesBackend::IdRole)) {
+                return static_cast<FlatpakResource*>(resource);
+            }
+        }
+    }
 
     g_autoptr(GError) error = nullptr;
     g_autoptr(FlatpakRemoteRef) remoteRef = nullptr;
@@ -387,7 +410,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
         remoteRef = flatpak_installation_install_ref_file (preferredInstallation(), bytes, m_cancellable, &error);
         if (!remoteRef) {
             qWarning() << "Failed to create install ref file:" << error->message;
-            const auto resources = resourcesByAppstreamName(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+            const auto resources = resourcesByAppstreamName(name);
             if (!resources.isEmpty()) {
                 return qobject_cast<FlatpakResource*>(resources.constFirst());
             }
@@ -404,7 +427,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
     asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
     asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
     asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
-    asComponent.setId(settings.value(QStringLiteral("Flatpak Ref/Name")).toString());
+    asComponent.setId(name);
 
     const QString iconUrl = settings.value(QStringLiteral("Flatpak Ref/Icon")).toString();
     if (!iconUrl.isEmpty()) {
@@ -424,7 +447,6 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
         auto installation = preferredInstallation();
         // We need to fetch metadata to find information about required runtime
         auto fw = new QFutureWatcher<QByteArray>(this);
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
         connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, installation, resource, fw, runtimeUrl]() {
             const auto metadata = fw->result();
             // Even when we failed to fetch information about runtime we still want to show the application
@@ -450,6 +472,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
             }
             fw->deleteLater();
         });
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
     } else {
         addResource(resource);
     }
@@ -521,6 +544,10 @@ void FlatpakBackend::addResource(FlatpakResource *resource)
     updateAppSize(installation, resource);
 
     m_resources.insert(resource->uniqueId(), resource);
+    if (!resource->extends().isEmpty()) {
+        m_extends.append(resource->extends());
+        m_extends.removeDuplicates();
+    }
 }
 
 class FlatpakSource
@@ -608,19 +635,7 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
     }
 
     auto fw = new QFutureWatcher<QList<AppStream::Component>>(this);
-    fw->setFuture(QtConcurrent::run(&m_threadPool, [appDirFileName]() -> QList<AppStream::Component> {
-        AppStream::Metadata metadata;
-        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
-        AppStream::Metadata::MetadataError error = metadata.parseFile(appDirFileName, AppStream::Metadata::FormatKindXml);
-        if (error != AppStream::Metadata::MetadataErrorNoError) {
-            qWarning() << "Failed to parse appstream metadata: " << error;
-            return {};
-        }
-
-        return metadata.components();
-    }));
     const auto sourceName = source.name();
-    acquireFetching(true);
     connect(fw, &QFutureWatcher<QList<AppStream::Component>>::finished, this, [this, fw, flatpakInstallation, appstreamIconsPath, sourceName]() {
         const auto components = fw->result();
         foreach (const AppStream::Component& appstreamComponent, components) {
@@ -637,6 +652,18 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
         acquireFetching(false);
         fw->deleteLater();
     });
+    acquireFetching(true);
+    fw->setFuture(QtConcurrent::run(&m_threadPool, [appDirFileName]() -> QList<AppStream::Component> {
+        AppStream::Metadata metadata;
+        metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
+        AppStream::Metadata::MetadataError error = metadata.parseFile(appDirFileName, AppStream::Metadata::FormatKindXml);
+        if (error != AppStream::Metadata::MetadataErrorNoError) {
+            qWarning() << "Failed to parse appstream metadata: " << error;
+            return {};
+        }
+
+        return metadata.components();
+    }));
 }
 
 void FlatpakBackend::loadInstalledApps()
@@ -684,9 +711,7 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
             continue;
         }
 
-        auto component = metadata.component();
-        component.setId(name + QLatin1String(".desktop"));
-        FlatpakResource *resource = new FlatpakResource(component, flatpakInstallation, this);
+        FlatpakResource *resource = new FlatpakResource(metadata.component(), flatpakInstallation, this);
 
         resource->setIconPath(pathExports);
         resource->setState(AbstractResource::Installed);
@@ -736,6 +761,11 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
 {
     auto fw = new QFutureWatcher<GPtrArray *>(this);
+    connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
+        auto refs = fw->result();
+        onFetchUpdatesFinished(installation, refs);
+        fw->deleteLater();
+    });
     fw->setFuture(QtConcurrent::run(&m_threadPool, [installation, this]() -> GPtrArray * {
         g_autoptr(GError) localError = nullptr;
         GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, m_cancellable, &localError);
@@ -744,11 +774,6 @@ void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
         }
         return refs;
     }));
-    connect(fw, &QFutureWatcher<GPtrArray *>::finished, this, [this, installation, fw](){
-        auto refs = fw->result();
-        onFetchUpdatesFinished(installation, refs);
-        fw->deleteLater();
-    });
 }
 
 void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstallation, GPtrArray *updates)
@@ -889,26 +914,23 @@ void FlatpakBackend::updateAppInstalledMetadata(FlatpakInstalledRef *installedRe
 
 bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation, FlatpakResource *resource)
 {
-    g_autoptr(GFile) installationPath = nullptr;
-
-    if (resource->type() != FlatpakResource::DesktopApp) {
+    if (resource->resourceType() != FlatpakResource::DesktopApp) {
         return true;
     }
 
-    installationPath = flatpak_installation_get_path(flatpakInstallation);
-    const QString path = QString::fromUtf8(g_file_get_path(installationPath)) + QStringLiteral("/app/%1/%2/%3/active/metadata").arg(resource->flatpakName()).arg(resource->arch()).arg(resource->branch());
+    const QString path = resource->installPath() + QStringLiteral("/metadata");
 
     if (QFile::exists(path)) {
         return updateAppMetadata(resource, path);
     } else {
         auto fw = new QFutureWatcher<QByteArray>(this);
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
         connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, flatpakInstallation, resource, fw]() {
             const auto metadata = fw->result();
             if (!metadata.isEmpty())
                 onFetchMetadataFinished(flatpakInstallation, resource, metadata);
             fw->deleteLater();
         });
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
 
         // Return false to indicate we cannot continue (right now used only in updateAppSize())
         return false;
@@ -979,7 +1001,7 @@ bool FlatpakBackend::updateAppSize(FlatpakInstallation *flatpakInstallation, Fla
 bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
 {
     // Calculate the runtime size
-    if (resource->state() == AbstractResource::None && resource->type() == FlatpakResource::DesktopApp) {
+    if (resource->state() == AbstractResource::None && resource->resourceType() == FlatpakResource::DesktopApp) {
         auto runtime = getRuntimeForApp(resource);
         if (runtime) {
             // Re-check runtime state if case a new one was created
@@ -1012,7 +1034,6 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
         }
 
         auto futureWatcher = new QFutureWatcher<FlatpakRunnables::SizeInformation>(this);
-        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
         connect(futureWatcher, &QFutureWatcher<FlatpakRunnables::SizeInformation>::finished, this, [this, resource, futureWatcher]() {
             auto value = futureWatcher->result();
             if (value.valid) {
@@ -1023,6 +1044,7 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
             }
             futureWatcher->deleteLater();
         });
+        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
     }
 
     return true;
@@ -1031,17 +1053,16 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
 void FlatpakBackend::onFetchSizeFinished(FlatpakResource *resource, guint64 downloadSize, guint64 installedSize)
 {
     FlatpakResource *runtime = nullptr;
-    if (resource->state() == AbstractResource::None && resource->type() == FlatpakResource::DesktopApp) {
+    if (resource->state() == AbstractResource::None && resource->resourceType() == FlatpakResource::DesktopApp) {
         runtime = getRuntimeForApp(resource);
     }
 
     if (runtime && !runtime->isInstalled()) {
         resource->setDownloadSize(runtime->downloadSize() + downloadSize);
-        resource->setInstalledSize(installedSize);
     } else {
         resource->setDownloadSize(downloadSize);
-        resource->setInstalledSize(installedSize);
     }
+    resource->setInstalledSize(installedSize);
 }
 
 void FlatpakBackend::updateAppState(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
@@ -1051,7 +1072,7 @@ void FlatpakBackend::updateAppState(FlatpakInstallation *flatpakInstallation, Fl
         // If the app is installed, we can set information about commit, arch etc.
         updateAppInstalledMetadata(ref, resource);
     } else {
-        // TODO check if the app is actuall still available
+        // TODO check if the app is actually still available
         resource->setState(AbstractResource::None);
     }
 }
@@ -1085,7 +1106,7 @@ bool FlatpakBackend::flatpakResourceLessThan(AbstractResource* l, AbstractResour
 
 ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &filter)
 {
-    if (filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakrepo")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakref"))) {
+    if (filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakrepo")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpakref")) || filter.resourceUrl.fileName().endsWith(QLatin1String(".flatpak"))) {
         auto stream = new ResultsStream(QStringLiteral("FlatpakStream-http-")+filter.resourceUrl.fileName());
 
         FlatpakFetchRemoteResourceJob *fetchResourceJob = new FlatpakFetchRemoteResourceJob(filter.resourceUrl, this);
@@ -1101,20 +1122,23 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
         return stream;
     } else if(filter.resourceUrl.scheme() == QLatin1String("appstream")) {
         return findResourceByPackageName(filter.resourceUrl);
-    } else if (!filter.resourceUrl.isEmpty() || !filter.extends.isEmpty())
+    } else if (!filter.resourceUrl.isEmpty() || (!filter.extends.isEmpty() && !m_extends.contains(filter.extends)))
         return new ResultsStream(QStringLiteral("FlatpakStream-void"), {});
 
     auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
     auto f = [this, stream, filter] () {
         QVector<AbstractResource*> ret;
-        foreach(AbstractResource* r, m_resources) {
-            if (r->isTechnical() && filter.state != AbstractResource::Upgradeable) {
+        foreach(auto r, m_resources) {
+            if (r->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable) {
                 continue;
             }
 
             if (r->state() < filter.state)
                 continue;
 
+            if (!filter.extends.isEmpty() && !r->extends().contains(filter.extends))
+                continue;
+
             if (filter.search.isEmpty() || r->name().contains(filter.search, Qt::CaseInsensitive) || r->comment().contains(filter.search, Qt::CaseInsensitive)) {
                 ret += r;
             }
@@ -1186,7 +1210,7 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
 
     FlatpakResource *resource = qobject_cast<FlatpakResource*>(app);
 
-    if (resource->type() == FlatpakResource::Source) {
+    if (resource->resourceType() == FlatpakResource::Source) {
         // Let source backend handle this
         FlatpakRemote *remote = m_sources->installSource(resource);
         if (remote) {
@@ -1199,35 +1223,10 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
         return nullptr;
     }
 
-    FlatpakJobTransaction *transaction = nullptr;
-    FlatpakInstallation *installation = resource->installation();
-
-    if (resource->propertyState(FlatpakResource::RequiredRuntime) == FlatpakResource::NotKnownYet && resource->type() == FlatpakResource::DesktopApp) {
-        transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole, true);
-        connect(resource, &FlatpakResource::propertyStateChanged, [resource, transaction, this] (FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState state) {
-            if (kind != FlatpakResource::RequiredRuntime) {
-                return;
-            }
-
-            if (state == FlatpakResource::AlreadyKnown) {
-                FlatpakResource *runtime = getRuntimeForApp(resource);
-                if (runtime && !runtime->isInstalled()) {
-                    transaction->setRuntime(runtime);
-                }
-            }
-            transaction->start();
-        });
-    } else {
-        FlatpakResource *runtime = getRuntimeForApp(resource);
-        if (runtime && !runtime->isInstalled()) {
-            transaction = new FlatpakJobTransaction(resource, runtime, Transaction::InstallRole);
-        } else {
-            transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
-        }
-    }
-
-    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, installation, resource] (Transaction::Status status) {
+    FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
+    connect(transaction, &FlatpakJobTransaction::statusChanged, [this, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
+            FlatpakInstallation *installation = resource->installation();
             updateAppState(installation, resource);
         }
     });
@@ -1243,7 +1242,7 @@ Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
 {
     FlatpakResource *resource = qobject_cast<FlatpakResource*>(app);
 
-    if (resource->type() == FlatpakResource::Source) {
+    if (resource->resourceType() == FlatpakResource::Source) {
         // Let source backend handle this
         if (m_sources->removeSource(resource->flatpakName())) {
             resource->setState(AbstractResource::None);
@@ -1273,24 +1272,6 @@ void FlatpakBackend::checkForUpdates()
     }
 }
 
-AbstractResource * FlatpakBackend::resourceForFile(const QUrl &url)
-{
-    if (!url.isLocalFile()) {
-        return nullptr;
-    }
-
-    FlatpakResource *resource = nullptr;
-    if (url.path().endsWith(QLatin1String(".flatpak"))) {
-        resource = addAppFromFlatpakBundle(url);
-    } else if (url.path().endsWith(QLatin1String(".flatpakref"))) {
-        resource = addAppFromFlatpakRef(url);
-    } else if (url.path().endsWith(QLatin1String(".flatpakrepo"))) {
-        resource = addSourceFromFlatpakRepo(url);
-    }
-
-    return resource;
-}
-
 QString FlatpakBackend::displayName() const
 {
     return QStringLiteral("Flatpak");
