@@ -21,6 +21,7 @@
 
 #include "FlatpakResource.h"
 #include "FlatpakBackend.h"
+#include "config-paths.h"
 
 #include <Transaction/AddonList.h>
 
@@ -31,6 +32,7 @@
 #include <KFormat>
 #include <KLocalizedString>
 
+#include <QProcess>
 #include <QDir>
 #include <QDebug>
 #include <QDesktopServices>
@@ -141,10 +143,17 @@ void FlatpakResource::updateFromRef(FlatpakRef* ref)
     setBranch(QString::fromUtf8(flatpak_ref_get_branch(ref)));
     setCommit(QString::fromUtf8(flatpak_ref_get_commit(ref)));
     setFlatpakName(QString::fromUtf8(flatpak_ref_get_name(ref)));
-    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? FlatpakResource::DesktopApp : FlatpakResource::Runtime);
+    setType(flatpak_ref_get_kind(ref) == FLATPAK_REF_KIND_APP ? DesktopApp :
+                                          extends().isEmpty() ? Runtime
+                                                              : Extension);
     setObjectName(packageName());
 }
 
+QString FlatpakResource::ref() const
+{
+    return typeAsString() + QLatin1Char('/') + flatpakName() + QLatin1Char('/') + arch() + QLatin1Char('/') + branch();
+}
+
 QStringList FlatpakResource::categories()
 {
     auto cats = m_appdata.categories();
@@ -237,9 +246,16 @@ int FlatpakResource::installedSize() const
     return m_installedSize;
 }
 
-bool FlatpakResource::isTechnical() const
+AbstractResource::Type FlatpakResource::type() const
 {
-    return m_id.type == FlatpakResource::Runtime;
+    switch (m_id.type) {
+        case FlatpakResource::Runtime:
+            return Technical;
+        case FlatpakResource::Extension:
+            return Addon;
+        default:
+            return Application;
+    }
 }
 
 QUrl FlatpakResource::homepage()
@@ -368,7 +384,7 @@ AbstractResource::State FlatpakResource::state()
     return m_state;
 }
 
-FlatpakResource::ResourceType FlatpakResource::type() const
+FlatpakResource::ResourceType FlatpakResource::resourceType() const
 {
     return m_id.type;
 }
@@ -376,11 +392,11 @@ FlatpakResource::ResourceType FlatpakResource::type() const
 QString FlatpakResource::typeAsString() const
 {
     switch (m_id.type) {
+        case FlatpakResource::Runtime:
+        case FlatpakResource::Extension:
+            return QLatin1String("runtime");
         case FlatpakResource::DesktopApp:
         case FlatpakResource::Source:
-            return QLatin1String("app");
-        case FlatpakResource::Runtime:
-            return QLatin1String("runtime");
         default:
             return QLatin1String("app");
     }
@@ -393,6 +409,13 @@ FlatpakResource::Id FlatpakResource::uniqueId() const
 
 void FlatpakResource::invokeApplication() const
 {
+    const QString desktopFile = installPath() + QStringLiteral("/export/share/applications/") + appstreamId();
+    const QString runservice = QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice");
+    if (QFile::exists(desktopFile) && QFile::exists(runservice)) {
+        QProcess::startDetached(runservice, {desktopFile});
+        return;
+    }
+
     g_autoptr(GCancellable) cancellable = g_cancellable_new();
     g_autoptr(GError) localError = nullptr;
 
@@ -524,6 +547,11 @@ QString FlatpakResource::installationPath(FlatpakInstallation* flatpakInstallati
     return QString::fromUtf8(g_file_get_path(path));
 }
 
+QString FlatpakResource::installPath() const
+{
+    return installationPath() + QStringLiteral("/app/%1/%2/%3/active").arg(flatpakName()).arg(arch()).arg(branch());
+}
+
 QUrl FlatpakResource::url() const
 {
     return m_resourceFile.isEmpty() ? QUrl(QStringLiteral("appstream://") + appstreamId()) : m_resourceFile;
@@ -543,3 +571,13 @@ QString FlatpakResource::sourceIcon() const
 {
     return QStringLiteral("flatpak-discover");
 }
+
+    QString FlatpakResource::author() const
+{
+    return m_appdata.developerName();
+}
+
+QStringList FlatpakResource::extends() const
+{
+    return m_appdata.extends();
+}
