@@ -131,6 +131,19 @@ QStandardItem * FlatpakSourcesBackend::sourceById(const QString& id) const
     return sourceIt;
 }
 
+QStandardItem * FlatpakSourcesBackend::sourceByUrl(const QString& url) const
+{
+    QStandardItem* sourceIt = nullptr;
+    for (int i = 0, c = m_sources->rowCount(); i<c; ++i) {
+        auto it = m_sources->item(i);
+        if (it->data(Qt::StatusTipRole) == url) {
+            sourceIt = it;
+            break;
+        }
+    }
+    return sourceIt;
+}
+
 bool FlatpakSourcesBackend::removeSource(const QString &id)
 {
     auto sourceIt = sourceById(id);
@@ -243,7 +256,8 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
 
     FlatpakSourceItem *it = new FlatpakSourceItem(!title.isEmpty() ? title : id);
     it->setCheckState(flatpak_remote_get_disabled(remote) ? Qt::Unchecked : Qt::Checked);
-    it->setData(remoteUrl.host(), Qt::ToolTipRole);
+    it->setData(remoteUrl.isLocalFile() ? remoteUrl.toLocalFile() : remoteUrl.host(), Qt::ToolTipRole);
+    it->setData(remoteUrl, Qt::StatusTipRole);
     it->setData(id, IdRole);
     it->setFlatpakInstallation(installation);
 
