@@ -85,7 +85,7 @@ void FwupdBackend::addResourceToList(FwupdResource* res)
 FwupdResource * FwupdBackend::createDevice(FwupdDevice *device)
 {
     const QString name = QString::fromUtf8(fwupd_device_get_name(device));
-    FwupdResource* res = new FwupdResource(name, this);
+    FwupdResource* res = new FwupdResource(name, nullptr);
 
     const QString deviceID = QString::fromUtf8(fwupd_device_get_id(device));
     res->setId(QStringLiteral("org.fwupd.%1.device").arg(QString(deviceID).replace(QLatin1Char('/'),QLatin1Char('_'))));
@@ -454,7 +454,7 @@ ResultsStream* FwupdBackend::search(const AbstractResourcesBackend::Filters& fil
     if (filter.resourceUrl.scheme() == QLatin1String("fwupd")) {
         return findResourceByPackageName(filter.resourceUrl);
     } else if (!filter.resourceUrl.isEmpty()) {
-        return new ResultsStream(QStringLiteral("FwupdStream-void"), {});
+        return resourceForFile(filter.resourceUrl);
     }
 
     auto stream = new ResultsStream(QStringLiteral("FwupdStream"));
@@ -518,7 +518,7 @@ Transaction* FwupdBackend::removeApplication(AbstractResource* /*app*/)
     return nullptr;
 }
 
-AbstractResource * FwupdBackend::resourceForFile(const QUrl& path)
+ResultsStream* FwupdBackend::resourceForFile(const QUrl& path)
 {
     g_autoptr(GError) error = nullptr;
 
@@ -544,13 +544,14 @@ AbstractResource * FwupdBackend::resourceForFile(const QUrl& path)
             }
             addResourceToList(app);
             connect(app, &FwupdResource::stateChanged, this, &FwupdBackend::updatesCountChanged);
+            return new ResultsStream(QStringLiteral("FwupdStream-file"), {app});
         }
         else
         {
             handleError(error);
         }
     }
-    return app;
+    return new ResultsStream(QStringLiteral("FwupdStream-void"), {});
 }
 
 QString FwupdBackend::displayName() const
