@@ -32,7 +32,7 @@ class AppPackageKitResource : public PackageKitResource
 
         QString appstreamId() const override;
 
-        bool isTechnical() const override;
+        AbstractResource::Type type() const override;
         QString name() const override;
         QVariant icon() const override;
         QStringList mimetypes() const override;
@@ -49,9 +49,12 @@ class AppPackageKitResource : public PackageKitResource
         QStringList extends() const override;
         void fetchScreenshots() override;
         void invokeApplication() const override;
-        bool canExecute() const override { return true; }
+        bool canExecute() const override;
         QDate releaseDate() const override;
         QString changelog() const override;
+        QString author() const override;
+
+        void fetchChangelog() override;
 
     private:
         const AppStream::Component m_appdata;
