@@ -21,13 +21,13 @@
 #include "PackageKitMessages.h"
 
 #include <PackageKit/Daemon>
-#ifdef PKQT_1_0
 #include <PackageKit/Offline>
-#endif
 #include <QDebug>
 #include <QAction>
 #include <QSet>
 
+#include <KSharedConfig>
+#include <KConfigGroup>
 #include <KLocalizedString>
 
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
@@ -69,6 +69,13 @@ PackageKitUpdater::~PackageKitUpdater()
 
 void PackageKitUpdater::prepare()
 {
+    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+        m_toUpgrade.clear();
+        m_allUpgradeable.clear();
+        enableNeedsReboot();
+        return;
+    }
+
     Q_ASSERT(!m_transaction);
     m_toUpgrade = m_backend->upgradeablePackages();
     m_allUpgradeable = m_toUpgrade;
@@ -156,14 +163,29 @@ void PackageKitUpdater::proceed()
 {
     if (!m_proceedFunctions.isEmpty())
         processProceedFunction();
-#ifdef PKQT_1_0
-    else if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE"))
+    else if (useOfflineUpdates())
         setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted | PackageKit::Transaction::TransactionFlagOnlyDownload);
-#endif
     else
         setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted);
 }
 
+bool PackageKitUpdater::useOfflineUpdates() const
+{
+    if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE"))
+        return true;
+    KConfigGroup group(KSharedConfig::openConfig(), "Software");
+    return group.readEntry<bool>("UseOfflineUpdates", false);
+}
+
+void PackageKitUpdater::setUseOfflineUpdates(bool use)
+{
+//     To enable from command line use:
+//     kwriteconfig5 --file discoverrc --group Software --key UseOfflineUpdates true
+
+    KConfigGroup group(KSharedConfig::openConfig(), "Software");
+    group.writeEntry<bool>("UseOfflineUpdates", use);
+}
+
 void PackageKitUpdater::start()
 {
     Q_ASSERT(!isProgressing());
@@ -201,13 +223,9 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
     m_backend->fetchUpdates();
     fetchLastUpdateTime();
 
-    if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE")) {
-#ifdef PKQT_1_0
+    if (useOfflineUpdates()) {
         PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
-        Q_EMIT passiveMessage(i18n("Please restart the computer to finish the installation"));
-#else
-        qWarning() << "PK_OFFLINE_UPDATE is set but discover was built against an old version of PackageKitQt that didn't support offline updates";
-#endif
+        Q_EMIT passiveMessage(i18n("Please restart the computer to finish the update"));
     }
 }
 
