@@ -41,6 +41,7 @@ class DISCOVERCOMMON_EXPORT ResourcesUpdatesModel : public QStandardItemModel
     Q_PROPERTY(QDateTime lastUpdate READ lastUpdate NOTIFY progressingChanged)
     Q_PROPERTY(qint64 secsToLastUpdate READ secsToLastUpdate NOTIFY progressingChanged)
     Q_PROPERTY(Transaction* transaction READ transaction NOTIFY progressingChanged)
+    Q_PROPERTY(bool needsReboot READ needsReboot NOTIFY needsRebootChanged)
     public:
         explicit ResourcesUpdatesModel(QObject* parent = nullptr);
 
@@ -57,6 +58,7 @@ class DISCOVERCOMMON_EXPORT ResourcesUpdatesModel : public QStandardItemModel
         qint64 secsToLastUpdate() const;
         QVector<AbstractBackendUpdater*> updaters() const { return m_updaters; }
         Transaction* transaction() const;
+        bool needsReboot() const;
 
     Q_SIGNALS:
         void downloadSpeedChanged();
@@ -64,6 +66,7 @@ class DISCOVERCOMMON_EXPORT ResourcesUpdatesModel : public QStandardItemModel
         void finished();
         void resourceProgressed(AbstractResource* resource, qreal progress);
         void passiveMessage(const QString &message);
+        void needsRebootChanged();
 
     public Q_SLOTS:
         void updateAll();
