@@ -36,6 +36,7 @@ Q_PROPERTY(QString iconName READ iconName NOTIFY updatesChanged)
 Q_PROPERTY(QString message READ message NOTIFY updatesChanged)
 Q_PROPERTY(QString extendedMessage READ extendedMessage NOTIFY updatesChanged)
 Q_PROPERTY(State state READ state NOTIFY updatesChanged)
+Q_PROPERTY(bool needsReboot READ needsReboot NOTIFY needsRebootChanged)
 public:
     enum State {
         NoUpdates,
@@ -60,6 +61,7 @@ public:
     uint securityUpdatesCount() const;
 
     QStringList loadedModules() const;
+    bool needsReboot() const { return m_needsReboot; }
 
 public Q_SLOTS:
     void configurationChanged();
@@ -67,14 +69,17 @@ public Q_SLOTS:
     void showDiscover();
     void showDiscoverUpdates();
     void showUpdatesNotification();
+    void reboot();
+    void foundUpgradeAction(UpgradeAction* action);
 
 Q_SIGNALS:
     void updatesChanged();
+    bool needsRebootChanged(bool needsReboot);
+    void newUpgradeAction(UpgradeAction* action);
 
 private:
     void showRebootNotification();
     void updateStatusNotifier();
-    void loadBackends();
 
     QList<BackendNotifierModule*> m_backends;
     bool m_verbose;
